<?php

/**
 Roomskills Local Connect Controller
 
 @Copyright 2021 Roomskills.com by ComputeQ UG
 @author Ingo Keutgen
 Mail: ingo.keutgen@computeq.co
 Web: http://www.roomskills.com
 
 based on the examples provide by MAX2PLAY

 */

class rlc_class_setup extends Service
{

    public $scriptPath = '';

    /**
     * Name of the process
     */
    protected $pname = 'roomskills_local_connect';

    /**
     * Name of the Process in the View-File (HTML-Content)
     */
    public $viewname = 'Roomskills-Local-Connect';

    public function __construct()
    {
        // Call Construct function from Service Class
        parent::__construct();
        
        // Set your Pluginname (for Navigation and Plugin-Configuration)
        $this->pluginname = _('RoomskillsLocalConnect');
        
        /**
         * Set Custom Translation File for Multilanguage
         * Use PO-Edit to parse your Plugin-folder for _t() Translation strings
         * Translation strings are translated with your custom .po files in folder /locale/[iso-lang-code]/LC_MESSAGES/translation.po
         */
        $this->registerLocale(dirname(__FILE__) . '/../locale', 'rlc');
        
        /**
         * Set Script Path to Plugin-Folder - create and Shell-Scripts for more Complex tasks
         */
        $this->scriptPath = dirname(__FILE__) . '/../scripts/';
        
        /**
         * set up handler for automatic Page refresh (message output for progess during complex tasks that take some time)
         */
        $this->checkAjaxCall('install');
        
        /**
         * If Button clicked or Form sent do Something
         * Parse Request Object for action variable
         */
        if (isset($_REQUEST['rlc_action'])) {
            
            if ($_REQUEST['rlc_action'] == 'install') {
                $this->rlc_install();
            }

            if ($_REQUEST['rlc_action'] == 'remove') {
                $this->rlc_remove();
            }

            /**
             * Start / Stop / Restart
             */
            if ($_REQUEST['rlc_action'] == 'start') {
                $this->rlc_start();
            }
            
            if ($_REQUEST['rlc_action'] == 'stop') {
                $this->rlc_stop();
            }
            
            if ($_REQUEST['rlc_action'] == 'restart') {
                $this->rlc_restart();
            }
        }
        
        // Check for running process of Processname
        $this->view->pid = $this->status($this->pname);
        
        // Get Debug Info
        $this->get_rlc_log();
        
        // Get Sidebar Infos
        $this->showHelpSidebar();
    }

    /**
     * Demo function to do Something
     * e.g.
     * Call Scripts and return Output to view
     */
    private function _doSomething()
    {
        $this->view->message[] = _('Do Something started');
        
        // #1 You can call scripts that do not need root-rights with:
        $output = nl2br(shell_exec('ls /opt/max2play/'));
        
        // Send Output to View-Script
        $this->view->message[] = 'Normalscript Output: ' . $output;
        
        // #2 If you want to run scripts with root rights do this:
        $output2 = nl2br($this->writeDynamicScript(array(
            'ls /opt/max2play/'
        )));
        
        $this->view->message[] = 'Rootscript Output: ' . $output2;
    }

    /**
     * install Roomskills_local_connect
     */
    private function rlc_install()
    {
        $this->view->message[] = "Roomskills Local Connect Install ...";
        
        // #2 If you want to run scripts with root rights do this:
        $inst_output = nl2br($this->writeDynamicScript(array(
            $this->scriptPath . "install.sh"
        )));
        
        $this->view->message[] = $inst_output;
    }

    /**
     * uninstall Roomskills_local_connect
     */
    private function rlc_remove()
    {
        $this->view->message[] = "Roomskills Local Connect Uninstall ...";
        
        // #2 If you want to run scripts with root rights do this:
        $inst_output = nl2br($this->writeDynamicScript(array(
            $this->scriptPath . "uninstall.sh"
        )));
        
        $this->view->message[] = $inst_output;
    }

    /**
     * stop Roomskills_local_connect service
     */
    private function rlc_stop()
    {
        $this->view->message[] = "Roomskills Local Connect Install ...";
        
        // #2 If you want to run scripts with root rights do this:
        $inst_output = nl2br($this->writeDynamicScript(array(
            $this->scriptPath . "stop.sh"
        )));
        
        $this->view->message[] = $inst_output;
    }

    /**
     * start Roomskills_local_connect service
     */
    private function rlc_start()
    {
        $this->view->message[] = "Roomskills Local Connect Install ...";
        
        // #2 If you want to run scripts with root rights do this:
        $inst_output = nl2br($this->writeDynamicScript(array(
            $this->scriptPath . "start.sh"
        )));
        
        $this->view->message[] = $inst_output;
    }

    /**
     * restart Roomskills_local_connect service
     */
    private function rlc_restart()
    {
        $this->view->message[] = "Roomskills Local Connect Install ...";
        
        // #2 If you want to run scripts with root rights do this:
        $inst_output = nl2br($this->writeDynamicScript(array(
            $this->scriptPath . "restart.sh"
        )));
        
        $this->view->message[] = $inst_output;
    }

    /**
     * Complex installer with Ajax Calls to get updates from running script
     * 
     * @param int $ajax
     * @return boolean
     */
    public function install($ajax = 0)
    {
        $progressfile = $this->scriptPath . 'install_progress.txt';
        if ($ajax == 0) {
            // First run - start Script and create temporary file for Output
            if ($this->getProgressWithAjax($progressfile, $create = 1, $reloadWhenFinished = 1, $lastlines = 0, $message = _t("Install Something Started"))) {
                $shellanswer = $this->writeDynamicScript(array(
                    $this->scriptPath . "install.sh >> " . $progressfile . " 2>&1 &"
                ), $background = false, $daemon = true);
            }
        } else {
            // Ajax Call every 3 seconds to get progression
            $status = $this->getProgressWithAjax($progressfile, $create = 0, $reloadWhenFinished = 1, $lastlines = 30);
            $this->view->message[] = nl2br($status);
            // Parse Output for String "finished"
            if (strpos($status, 'finished') !== FALSE) {
                // Finished Progress - did not delete progressfile yet
                $this->view->message[] = _t('Installation finished');
                // Delete Progressfile
                shell_exec('rm ' . $progressfile);
            }
        }
        return true;
    }

    /**
     * get some Debug Output and save it for view
     */
    private function get_rlc_log()
    {
        $out['Process'] = shell_exec('ps -ef | grep /opt/roomskills_local_connect/roomskills_local_connect | grep -v grep');
        $out['Log File'] = shell_exec('cat /opt/roomskills_local_connect/roomskills_local_connect.log');
        $this->view->rlc_log = $out;
    }

    public function showHelpSidebar()
    {
        global $helpSidebar;
        $language =substr($_SERVER['HTTP_ACCEPT_LANGUAGE'], 0, 2);
        // look through sorted list and use first one that matches our languages
        if ($language  == 'de') {
            $lang_help_title = "Roomskills Local Connect MAX2PLAY Plugin Hilfe";
            $lang_help_content = "<ul><li>Diese Plugin dient der Anbindung lokaler Systeme wie dem Logitech Mediaserver an den Roomskills.com Service</li><li>Mit Roomskills MyMusic kann man Musik vom Logitech Mediaserver auf Deinen Echo Lautsprechen abspielen</li><li>Mit Roomskills Music Control LMS kann man Deine Logitech Mediaserver Player mit Alexa steuern.</li><li>Nähere infos unter <a href='https://www.roomskills.com' target='rlc'>https://www.roomskills.com</a></li></ul>";
            $lang_help_link = "https://www.roomskills.com";
        } else {
            $lang_help_title = "Roomskills Local Connect MAX2PLAY Plugin help";
            $lang_help_content = "<ul><li>This Plugin is used to connect your local devices like the Logitech Mediaserver to the Roomskills.com Service</li><li>With Roomskills MyMusic you can play music from your Logitech Mediaserver on your echo speakers.</li><li>With Roomskills Music Control LMS you can control your Logitech Mediaserver player with Alexa.</li><li>More info at <a href='https://www.roomskills.com' target='rlc'>https://www.roomskills.com</a></li></ul>";
            $lang_help_link = "https://www.roomskills.com";
        }
        
        
        $helpSidebar['title'] = $lang_help_title;
        $helpSidebar['content'] = $lang_help_content;
        $helpSidebar['link'] = $lang_help_link;
        return true;
    }
}

// Create an instance of your Class
$RoomskillsLocalConnect = new rlc_class_setup();

// This Line includes the View-Script -> it should have the same name as your class
include_once (dirname(__FILE__) . '/../view/setup.php');

